@@ -1,5 +1,5 @@
 /* This file is part of the yazpp toolkit.
- * Copyright (C) 1998-2012 Index Data and Mike Taylor
+ * Copyright (C) 1998-2013 Index Data and Mike Taylor
  * All rights reserved.
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions are met:
@@ -40,11 +40,10 @@ namespace yazpp_1 {
         void cleanup(bool all);
         void set_period(int sec);
     private:
+        struct Rep;
         struct Peer;
-        
-        int m_period; 
-        Peer *m_peers;
-        Peer **lookup(const char *peername);
+
+        Rep *m_p;
     };
 }
 
