@@ -1,5 +1,5 @@
 /* This file is part of the yazpp toolkit.
- * Copyright (C) 1998-2012 Index Data and Mike Taylor
+ * Copyright (C) 1998-2013 Index Data and Mike Taylor
  * All rights reserved.
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions are met:
@@ -29,7 +29,6 @@
 #ifndef YAZPP_TIMESTAT_H
 #define YAZPP_TIMESTAT_H
 
-#include <time.h>
 #include <yaz/yconfig.h>
 
 namespace yazpp_1 {
@@ -40,10 +39,8 @@ namespace yazpp_1 {
         void add_bytes(int m);
         int get_total();
     private:
-        time_t m_sec;   // time of most recent bucket
-        int *m_bucket;
-        int m_ptr;
-        int m_size;
+        class Rep;
+        Rep *m_p;
     };
 }
 
