@@ -1,5 +1,5 @@
 /* This file is part of the yazpp toolkit.
- * Copyright (C) 1998-2012 Index Data and Mike Taylor
+ * Copyright (C) 1998-2013 Index Data and Mike Taylor
  * All rights reserved.
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions are met:
@@ -58,11 +58,8 @@ class YAZ_EXPORT Yaz_Z_Query : public Yaz_Query {
     /// Assign RPN string to it
     Yaz_Z_Query& operator=(const char *rpn);
  private:
-    char *m_buf;
-    int m_len;
-    ODR odr_decode;
-    ODR odr_encode;
-    ODR odr_print;
+    class Rep;
+    Rep *m_p;
 };
 };
 #endif
