@@ -1,5 +1,5 @@
 /* This file is part of the yazpp toolkit.
- * Copyright (C) 1998-2012 Index Data and Mike Taylor
+ * Copyright (C) 1998-2013 Index Data and Mike Taylor
  * See the file LICENSE for details.
  */
 
@@ -193,16 +193,17 @@ int main(int argc, char **argv)
     char *arg;
     char *prog = *argv;
     const char *addr = "tcp:@:9999";
+    const char *cert_fname = 0;
     char *apdu_log = 0;
-    
+
     SocketManager mySocketManager;
-    
+
     PDU_Assoc *my_PDU_Assoc = 0;
-    
+
     MyServer *z = 0;
     int ret;
-    
-    while ((ret = options("a:v:T", argv, argc, &arg)) != -2)
+
+    while ((ret = options("a:C:v:T", argv, argc, &arg)) != -2)
     {
         switch (ret)
         {
@@ -212,6 +213,9 @@ int main(int argc, char **argv)
         case 'a':
             apdu_log = xstrdup(arg);
             break;
+        case 'C':
+            cert_fname = xstrdup(arg);
+            break;
         case 'v':
             yaz_log_init_level (yaz_log_mask_str(arg));
             break;
@@ -231,7 +235,9 @@ int main(int argc, char **argv)
 #else
     my_PDU_Assoc = new PDU_Assoc(&mySocketManager);
 #endif
-    
+
+    my_PDU_Assoc->set_cert_fname(cert_fname);
+
     z = new MyServer(my_PDU_Assoc);
     z->server(addr);
     if (apdu_log)
