@@ -1,5 +1,5 @@
 /* This file is part of the yazpp toolkit.
- * Copyright (C) 1998-2012 Index Data and Mike Taylor
+ * Copyright (C) 1998-2013 Index Data and Mike Taylor
  * See the file LICENSE for details.
  */
 
@@ -47,7 +47,12 @@ PDU_AssocThread::PDU_AssocThread(
     ISocketObservable *socketObservable)
     : PDU_Assoc(socketObservable)
 {
-    
+
+}
+
+PDU_AssocThread::~PDU_AssocThread()
+{
+
 }
 
 void worker::run()
@@ -64,7 +69,7 @@ void worker::run()
 void __cdecl
 #else
 void *
-#endif 
+#endif
 events(void *p)
 {
     worker *w = (worker *) p;
@@ -83,7 +88,7 @@ void PDU_AssocThread::childNotify(COMSTACK cs)
     /// Clone PDU Observer
     new_observable->m_PDU_Observer =
         m_PDU_Observer->sessionNotify(new_observable, cs_fileno(cs));
-    
+
     if (!new_observable->m_PDU_Observer)
     {
         new_observable->shutdown();
