@@ -1,5 +1,5 @@
 /* This file is part of the yazpp toolkit.
- * Copyright (C) 1998-2012 Index Data and Mike Taylor
+ * Copyright (C) 1998-2013 Index Data and Mike Taylor
  * See the file LICENSE for details.
  */
 
@@ -35,9 +35,9 @@ private:
     int m_test_reported;
 };
 
-Zlint::Zlint(IPDU_Observable *the_PDU_Observable) : 
+Zlint::Zlint(IPDU_Observable *the_PDU_Observable) :
     Z_Assoc(the_PDU_Observable)
-    
+
 {
     m_PDU_Observable = the_PDU_Observable;
     m_host = 0;
@@ -79,13 +79,12 @@ void Zlint::timeoutNotify()
     {
         if (m_cur_test->m_t->recv_fail(this, 2) != TEST_FINISHED)
         {
-            close();
             client(m_host);
             timeout(30);
             return;
         }
+        close_goto_next();
     }
-    close_goto_next();
 }
 
 void Zlint::failNotify()
@@ -94,13 +93,12 @@ void Zlint::failNotify()
     {
         if (m_cur_test->m_t->recv_fail(this, 1) != TEST_FINISHED)
         {
-            close();
             client(m_host);
             timeout(30);
             return;
         }
+        close_goto_next();
     }
-    close_goto_next();
 }
 
 void Zlint::connectNotify()
@@ -109,8 +107,8 @@ void Zlint::connectNotify()
     {
         if (m_cur_test->m_t->init(this) != TEST_FINISHED)
             return;
+        close_goto_next();
     }
-    close_goto_next();
 }
 
 void Zlint::recv_GDU(Z_GDU *gdu, int len)
@@ -122,23 +120,25 @@ void Zlint::recv_GDU(Z_GDU *gdu, int len)
             return;
         if (r == TEST_REOPEN)
         {
-            close();
             client(m_host);
             timeout(30);
             return;
         }
+        close_goto_next();
     }
-    close_goto_next();
 }
 
 void Zlint::close_goto_next()
 {
-    close();
     if (m_cur_test)
         m_cur_test = m_cur_test->m_next;
     if (m_cur_test)
+    {
         client(m_host);
-    timeout(30);
+        timeout(30);
+    }
+    else
+        close();
 }
 
 IPDU_Observer *Zlint::sessionNotify(
@@ -149,7 +149,7 @@ IPDU_Observer *Zlint::sessionNotify(
 
 Z_ReferenceId *Zlint::mk_refid(const char *buf, int len)
 {
-    Z_ReferenceId *id = 
+    Z_ReferenceId *id =
         (Z_ReferenceId *) odr_malloc(odr_encode(), sizeof(*id));
     id->size = id->len = len;
     id->buf = (unsigned char*) odr_malloc(odr_encode(), len);
@@ -160,15 +160,12 @@ Z_ReferenceId *Zlint::mk_refid(const char *buf, int len)
 int Zlint::initResponseGetVersion(Z_InitResponse *init)
 {
     int no = 0;
-    int off = 0;
     int i;
     for (i = 0; i<12; i++)
         if (ODR_MASK_GET(init->protocolVersion, no))
         {
             no = i+1;
         }
-        else
-            off = 1;
     return no;
 }
 
